function varargout=bimoc_system(what,varargin);
% Implementation of Bimanual optimal control (BIMOC) simulation for the
% paper "Optimal task dependent changes of feedback control and
% adaptation" (Diedrichsen, 2007). 
% 
% Bimoc_system generates a structure that contains 
% a. The system of the two arms, using cetain parameters 
% b. A cost function of the system under certain task constraints 
% 
% Joern Diedrichsen, 2007, j.diedrichsen@bangor.ac.uk 

% Parameters for the system 
delta_t=1/200;                  % Time step in s 
statenoise=1;                   % Assumed noise in the system, as in [1]
measurenoise=[0.02;0.05;1];      % Measurement noise for position, velocity and force, as in [1]
tau1=0.04;                      % Muscle system, first time constant, as in [1]
tau2=0.04;                      % Muscle system, second time constant, as in [1]
m=0.5;                          % Mass of hand at endpoint, derived by local linear approximation from [2]
spring_x=124;                   % Stiffness of the hand system, fitted to experimental data 
tau_sense=0.015;                % Time constant of sensory delay filter 
num_sense=4;                    % Order of sensory delay filter 

% Constants for costfunction 
MT=0.9;                           % Absolute maximal movement time 
control_cost=2e-5;             % Control cost  
pos_cost=40;                   % Cost for position 
vel_cost=5;                   % Cost for velocity 
tau_cost=0.051;                    % Increase in position and velocity cost, fitted to experimental data 
amplitude=0.1;                  % Amplitude of the movement 

% resultant state vector is 
% 01: x-Position left hand [m]
% 02: x-velocity left hand [m/s]
% 03: x-force left hand [N]
% 04: x-muscle activation left hand [arbitrary units]
% 05: y-Position left hand 
% 06: y-velocity left hand 
% 07: y-force left hand 
% 08: y-muscle activation left hand 
% 09: x-Position right hand 
% 10: x-velocity right  hand 
% 11: x-force right  hand 
% 12: x-muscle activation right  hand 
% 13: y-Position right  hand 
% 14: y-velocity right  hand 
% 15: y-force right  hand 
% 16: y-muscle activation right  hand 
% 17: x-position target left 
% 18: y-position target left 
% 19: x-position target right 
% 20: y-position target right 
% DELAY FILTERS FOR SENSORY SYSTEM 
% 21-48: in the sequence of [px, vx, fx, py, vy, fy] and filter order 1-4 for each hand 
switch(what)
    % -----------------------------------------
    % generate the system 
    case 'system'
        vararginoptions(varargin,{'m','tau1','tau2','delta_t','amplitude','measurenoise','spring_x'});
        
        % Make Matrix A for differental equation. x_dot = A * x, for one
        % hand, one coordinate: x=[p v f g]'
        Diff_A=[0           1                 0             0    ;...
                0           0                1/m            0    ;...
                0           0                -1/tau2          1/tau2    ;...
                0           0                 0             -1/tau1];            

        % From this make Matrix A for the full system for difference equation x_n+1 = A * x_n 
        % Full state vector is: [Left_x Left_y Right_x Right_y];
        M.A=blockdiag(eye(4)+delta_t*Diff_A,eye(4)+delta_t*Diff_A,eye(4)+delta_t*Diff_A,eye(4)+delta_t*Diff_A,eye(4));

        % Add critically damped spring in the x-direction for perturbed
        % hand 
        M.A(2,1)=-spring_x*delta_t;  
        M.A(2,17)=spring_x*delta_t;
        
        % Make matrix B for one arm/coordinate: U influences only muscle
        % activation 
        diff_B=[0;0;0;1/tau1];
        % And make B for difference equation for the full system 
        M.B=blockdiag(diff_B*delta_t,diff_B*delta_t,diff_B*delta_t,diff_B*delta_t);
        M.B=[M.B;zeros(4,4)];
        
        % Make Observation Matrix H: Position, velocity and force can be
        % observed 
        H=[eye(3) zeros(3,1)];
        M.H=blockdiag(H,H,H,H);M.H=[M.H zeros(12,4)];
        M.os=size(M.H,1);       % Size of the observation vector: 4
        M.ss=size(M.A,1);       % Size of the state vector: 12 
        M.x0=[-.06;zeros(7,1);.06;zeros(7,1);-.06;amplitude;.06;amplitude]; % Initial state set target to 
        M.Q=eye(M.ss)*statenoise;
        M.R=diag(repmat(measurenoise,4,1));
        M.delta_t=delta_t;

        % Now delay the sensory system 
        M=oc_delaysystem('exp_filter',M,'tau',tau_sense,'num',num_sense);
        varargout={M};
        
    % -----------------------------------------
    % Add a viscous curl field to the system 
    case 'forcefield'
        M=varargin{1}; 
        forcefield=varargin{2};
        M.A(2,2+4,:)=forcefield(1)*delta_t;
        if (length(forcefield)>1)
            M.A(2+8,2+4+8,:)=forcefield(2)*delta_t;
        end;
        varargout={M};

    % -----------------------------------------
    % generate the cost function under one or two cursor condition 
    % Cost function is written as sum(n=1:N) [xn ' * Qn * x + un' * R * un]
    case 'costfunction'
        M=varargin{1};
        vararginoptions({varargin{2:end}},{'MT','control_cost','pos_cost','vel_cost','tau','cursor'});

        % Initialize cost function matrix 
        J.Q=zeros(size(M.A,1),size(M.A,2),max(MT)/M.delta_t);
        J.R=eye(size(M.B,2))*control_cost*M.delta_t;
        
        % Now build the elements of the cost 
        v=[0;1;0;0]*sqrt(vel_cost);
        C=v*v';
        C=blockdiag(C,C,C,C,zeros(4,4));
        switch (cursor)
            case 2 
            p=[1;0;0;0];
            p1=[p;zeros(12,1);-1;0;0;0]*sqrt(pos_cost);
            p2=[zeros(4,1);p;zeros(8,1);0;-1;0;0]*sqrt(pos_cost);
            p3=[zeros(8,1);p;zeros(4,1);0;0;-1;0]*sqrt(pos_cost);
            p4=[zeros(12,1);p;0;0;0;-1]*sqrt(pos_cost);
            C=C+p1*p1'+p2*p2'+p3*p3'+p4*p4';
            case 1
            p=[0.5;0;0;0];p0=[0;0;0;0];
            p1=[p;p0;p;p0;-0.5;0;-0.5;0]*sqrt(pos_cost);
            p2=[p0;p;p0;p;0;-0.5;0;-0.5]*sqrt(pos_cost);
            C=C+p1*p1'+p2*p2';
        end;

        % Now ramp the position and velocity cost up during the movement 
        x=[MT-M.delta_t:-M.delta_t:0]; 
        w=exp(-x./tau_cost);w=w./sum(w);
        n=size(C,1); 
        for i=1:size(J.Q,3);
            J.Q(1:n,1:n,i)=J.Q(1:n,1:n,i)+w(i)*C;              
        end;
        varargout={J};    
end;
