function [x,u,xnn]=oc_runsystem(M,V,J,varargin); 
% function [x,u]=oc_runsystem(M,V,varargin); 
% Simulates a system M under control gains V 
% INPUT 
% M: system-structure with matrices A,B,H,Q,R 
% V: time-dependent cost to go matrix 
% J: cost function (just need for matrix R) 
% OUTPUT
% x: state vectors x_1....x_N 
% u: motor commands u_1....u_N 
% xnn: state estimate xhat_1...xhat_N 
% VARARGIN: 
% 'plant',M     : define a new system (M) as plant, if forward model and system
%                are different
% 'v',vec       : Added noise to the system 
% 'w',vec       : Added noise to the observations 
% 'signalnoise',k: Adds signal depedent noise, SD = k * abs(u) 
% Joern Diedrichsen 2007 j.diedrichsen@bangor.ac.uk 
plant=[];
fig=0;
x=M.x0;u=[];
xnpn=M.x0;Pnpn=M.Q;
N=size(V,3);
control_index=[1:N];
v=zeros(M.ss,N);
w=zeros(M.os,N);
signalnoise=0;
Jacobian=[];

vararginoptions(varargin,{'control_index','v','w','plant','signalnoise','Jacobian'});

% Define the system for the plant: identical to the forward model (M),
% unless there are fields defined in the 'plant' 
Mplant=M;
if (isstruct(plant))
    F=fieldnames(plant);
    for i=1:length(F)
        Mplant.(F{i})=plant.(F{i});
    end;
end;

% Make all system matrices 3-dimensional to deal with possibly time-varying
% systems (important to implement iLQG) 
M=oc_expandsystem(M,N);
Mplant=oc_expandsystem(Mplant,N);

% Run the system from n+1 to N 
for n=1:N-1
    c=control_index(n);
    % Calculate the measurement, add mesurement noise vector 
    y(:,n)=M.H*x(:,n)+w(:,n);

    % Measurement update 
    K(:,:,n)=Pnpn(:,:,n)*M.H'*inv(M.H*Pnpn(:,:,n)*M.H'+M.R);
    xnn(:,n)=xnpn(:,n)+K(:,:,n)*(y(:,n)-M.H*xnpn(:,n));
    Pnn(:,:,n)=Pnpn(:,:,n)-K(:,:,n)*M.H*Pnpn(:,:,n);
    
    % Calculating control signal 
    u(:,n)=-(inv(M.B(:,:,n)'*V(:,:,c+1)*M.B(:,:,n)+J.R)*M.B(:,:,n)'*(V(:,:,c+1)*M.A(:,:,n)*xnn(:,n)));
    
    % Time update: Forward model prediction 
    xnpn(:,n+1)=M.A(:,:,n)*xnn(:,n)+M.B(:,:,n)*u(:,n);
    Pnpn(:,:,n+1)=M.A(:,:,n)*Pnn(:,:,n)*M.A(:,:,n)'+M.Q;
    
    % Time update: real system. If field is a function handle, the system
    % is non-linear and state dependent 
    if (isa(Mplant.A,'function_handle'))
        A=feval(Mplant.A,x(:,n),Mplant);
    else 
        A=Mplant.A(:,:,n);
    end;
    if (isa(Mplant.B,'function_handle'))
        B=feval(Mplant.B,x(:,n),Mplant);
    else 
        B=Mplant.B(:,:,n);
    end;
    % Now add the noise to system update 
    if (signalnoise==0) 
        x(:,n+1)=[A*x(:,n)+B*u(:,n)+v(:,n)];
    elseif (~isempty(Jacobian))
        if (length(size(Jacobian))==2)
            Jac=Jacobian;
        else 
            Jac=Jacobian(:,:,n);
        end;
        Jacinv=inv(Jac');
        S=Jacinv*(diag((Jac'*u(:,n)).^2))*Jacinv'*signalnoise.^2;
        u(:,n)=u(:,n)+mvnrnd(zeros(size(u,1),1),S)';
        x(:,n+1)=[A*x(:,n)+B*u(:,n)+v(:,n)];
    elseif (isscalar(signalnoise)) 
        u(:,n)=u(:,n)+normrnd(0,abs(u(:,n))*signalnoise);
        x(:,n+1)=[A*x(:,n)+B*u(:,n)+v(:,n)];        
    elseif(isnumeric(signalnoise))
        S=(diag((signalnoise*u(:,n)).^2));
        u(:,n)=u(:,n)+mvnrnd(zeros(size(u,1),1),S)';
        x(:,n+1)=[A*x(:,n)+B*u(:,n)+v(:,n)];                
    end;
end;